unit ImagesCache;

interface

uses
  Windows,
  Classes,
  Controls,
  ImgList,
  Graphics,

  IntegerList,
  PluginIntf,
  PluginIntfWrapper;

type
  TNamedImageList = class(TImageList)
  private
    FIds: TIntegerList;
    FNames: TStringList;
    function GetNames(Index: Integer): String;
  public
    constructor CreateSize(AWidth, AHeight: Integer);
    destructor Destroy; override;
    function AddMaskedByName(AImage: TBitmap; AMaskColor: TColor; AId: Integer;
      const AName: string): Integer;
    function GetIndex(const AName: string): Integer; overload;
    function GetIndex(AId: Integer): Integer; overload;
    property Names[Index: Integer]: string read GetNames;
  end;

  TImagesCache = class
  private
    FAppProvider: TAppProvider;
    F_ilSmallTypes: TNamedImageList;
    F_ilLargeTypes: TImageList;
    procedure LoadSmallTypesImages;
    procedure LoadLargeTypeImages;
  public
    constructor Create(AAppProvider: TAppProvider);
    destructor Destroy; override;
    property ilSmallTypes: TNamedImageList read F_ilSmallTypes;
    property ilLargeTypes: TImageList read F_ilLargeTypes;
  end;


implementation

uses
  SysUtils,

  MiscGUI;


{ TImagesCache }

constructor TImagesCache.Create(AAppProvider: TAppProvider);
begin
  FAppProvider := AAppProvider;

  F_ilSmallTypes := TNamedImageList.CreateSize(16, 16);
  ConvertToHighColor(F_ilSmallTypes);

  F_ilLargeTypes := TImageList.CreateSize(32, 32);
  ConvertToHighColor(F_ilLargeTypes);

  LoadSmallTypesImages;
  LoadLargeTypeImages;
end;

destructor TImagesCache.Destroy;
begin
  FreeAndNil(F_ilLargeTypes);
  FreeAndNil(F_ilSmallTypes);
  inherited;
end;

procedure TImagesCache.LoadSmallTypesImages;
var
  ds: IPluginDataSet;
  Bitmap: TBitmap;
  p: TStringStream;
begin
  ilSmallTypes.Clear;
  ds := FAppProvider.AppCache.GetTypes;

  Bitmap := TBitmap.Create;
  Bitmap.PixelFormat := pf24bit;
  Bitmap.Transparent := True;
  try
    ds.First;
    while not ds.Eof do
    begin
      p := TStringStream.Create(ds.AsString('_ICON'));
      Bitmap.LoadFromStream(p);
      StretchBitMap(Bitmap, ilSmallTypes.Width, ilSmallTypes.Height);
      ilSmallTypes.AddMaskedByName(Bitmap,
                Bitmap.TransparentColor,
                ds.AsInteger('_ID'),
                ds.AsString('_TYPENAME'));
      ds.Next;
    end;
  finally
    FreeAndNil(Bitmap);
  end;
end;

procedure TImagesCache.LoadLargeTypeImages;
var
  i: Integer;
  ds: IPluginDataSet;
  Bitmap: TBitmap;
begin
  ilLargeTypes.Clear;

  ds := FAppProvider.AppCache.GetTypes;
  for i := 0 to ilSmallTypes.Count - 1 do
    if ds.Locate('_TYPENAME', ilSmallTypes.Names[i]) then
    begin
      Bitmap := TBitmap.Create;
      Bitmap.PixelFormat := pf32bit;
      Bitmap.Transparent := True;
      try
        if ds.AsInteger('_DOCUMENT') = 1 then
        begin
          ilSmallTypes.GetBitmap(i, Bitmap);
          StretchBitMap(Bitmap, ilLargeTypes.Width, ilLargeTypes.Height);
          ilLargeTypes.AddMasked(Bitmap, Bitmap.TransparentColor);
        end
        else
        begin
          Bitmap.SetSize(ilLargeTypes.Width, ilLargeTypes.Height);
          BitmapLoadFromResource(HInstance, 'IMAGE_FOLDER_32', RT_RCDATA, Bitmap);

          ilSmallTypes.Draw(Bitmap.Canvas,
                     ilLargeTypes.Width - ilSmallTypes.Width,
                     ilLargeTypes.Height - ilSmallTypes.Height,
                     i);
          ilLargeTypes.Add(Bitmap, nil);
        end;
      finally
        FreeAndNil(Bitmap);
      end;
    end;
end;


{ TNamedImageList }

constructor TNamedImageList.CreateSize(AWidth, AHeight: Integer);
begin
  inherited CreateSize(AWidth, AHeight);
  FIds := TIntegerList.Create;
  FNames := TStringList.Create;
end;

destructor TNamedImageList.Destroy;
begin
  FreeAndNil(FIds);
  FreeAndNil(FNames);
  inherited;
end;

function TNamedImageList.AddMaskedByName(AImage: TBitmap; AMaskColor: TColor;
  AId: Integer; const AName: string): Integer;
begin
  Result := Self.AddMasked(AImage, AMaskColor);
  FIds.Add(AId);
  FNames.Add(AName);
end;

function TNamedImageList.GetIndex(const AName: string): Integer;
begin
  Result := FNames.IndexOf(AName);
end;

function TNamedImageList.GetIndex(AId: Integer): Integer;
begin
  Result := FIds.IndexOf(AId);
end;

function TNamedImageList.GetNames(Index: Integer): String;
begin
  Result := FNames[Index];
end;

end.
