unit IntegerList;
//  TIntegerList      Integer

interface

uses
  Classes, SysUtils;

type
  EIntegerListException = class(Exception);

  TIntegerList = class
  protected
    FList: TList;
    FSorted: Boolean;
    FAcceptDuplicates: Boolean;
    procedure Error(Msg: String);
    function GetCount: Integer;
    function GetItems(Index: Integer): Integer;
    procedure SetItems(Index: Integer; ACont: Integer); virtual;
  private
    procedure RemoveDuplicates;
    procedure SetAcceptDuplicates(const Value: Boolean);
  public
    constructor Create(ASorted: Boolean = false; AAcceptDuplicates: Boolean = False); 
    destructor Destroy; override;
    function Find(const ACont: Integer; var Index: Integer): Boolean;
    function Add(ACont: Integer): Integer; virtual;
    procedure AddItems(AddList: TIntegerList);
    procedure Insert(Index: Integer; ACont: Integer);
    procedure Delete(Index: Integer);
    procedure Clear;
    function Extract(Item: Integer): Integer; virtual;
    function Remove(ACont: Integer): Integer; virtual;
    function First: Integer;
    function Last: Integer;
    function IndexOf(ACont: Integer): Integer;
    function HaveItem(ACont: Integer): Boolean;
    function GetAsString(stDelimiter: String = ','): String;
    function GetAsVarArray: Variant;
    procedure SetAsString(stValues: String; stDelimiter: String = ',');
    property Items[Index: Integer]: Integer read GetItems write SetItems; default;
    property Sorted: Boolean read FSorted;
    property AcceptDuplicates: Boolean read FAcceptDuplicates write SetAcceptDuplicates;
    property Count: Integer read GetCount;
  end;

implementation

uses
  StrUtils,
  Variants;


constructor TIntegerList.Create(ASorted: Boolean; AAcceptDuplicates: Boolean);
begin
  inherited Create;
  FList := TList.Create;
  FSorted := ASorted;
  FAcceptDuplicates := AAcceptDuplicates;
end;

destructor TIntegerList.Destroy;
begin
  FList.Free;
  inherited;
end;

procedure TIntegerList.Error(Msg: String);
begin
  raise EIntegerListException.Create(Msg);
end;

function TIntegerList.GetCount: Integer;
begin
  Result := FList.Count;
end;

function TIntegerList.Find(const ACont: Integer; var Index: Integer): Boolean;
var
  L, H, I: Integer;
begin
  Result := False;
  L := 0;
  H := Count - 1;
  while L <= H do
  begin
    I := (L + H) shr 1;
    if Items[I] < ACont then L := I + 1 else
    begin
      H := I - 1;
      if Items[I] = ACont then
      begin
        Result := True;
        if not FAcceptDuplicates then L := I;
      end;
    end;
  end;
  Index := L;
end;

function TIntegerList.GetItems(Index: Integer): Integer;
begin
  Result := Integer(FList[Index]);
end;

procedure TIntegerList.SetItems(Index: Integer; ACont: Integer);
begin
  if Sorted then Error('    ');
  FList.Items[Index] := Pointer(ACont);
end;

function TIntegerList.Add(ACont: Integer): Integer;
begin
  if not Sorted then
    Result := FList.Count
  else
    if Find(ACont, Result) then
      if not FAcceptDuplicates then
        Exit;
  FList.Insert(Result, Pointer(ACont));
end;

procedure TIntegerList.Insert(Index: Integer; ACont: Integer);
begin
  if Sorted then Error('    ');
  FList.Insert(Index, Pointer(ACont));
end;

procedure TIntegerList.Delete(Index: Integer);
begin
  FList.Delete(Index);
end;

procedure TIntegerList.AddItems(AddList: TIntegerList);
var
  I: Integer;
begin
  for I:=0 to AddList.Count-1 do
    Add(AddList[I]);
end;

procedure TIntegerList.Clear;
begin
  FList.Clear;
end;

function TIntegerList.Extract(Item: Integer): Integer;
begin
  Result := IndexOf(Item);
  if Result >= 0 then
    FList.Delete(Result);
end;

function TIntegerList.Remove(ACont: Integer): Integer;
begin
  Result := IndexOf(ACont);
  if Result >= 0 then
    FList.Delete(Result);
end;

function TIntegerList.First: Integer;
begin
  Result := Integer(FList.First);
end;

function TIntegerList.Last: Integer;
begin
  Result := Integer(FList.Last);
end;

function TIntegerList.IndexOf(ACont: Integer): Integer;
begin
  if not Sorted then
    Result := FList.IndexOf(Pointer(ACont))
  else
  if not Find(ACont, Result) then Result := -1;
end;

function TIntegerList.HaveItem(ACont: Integer): Boolean;
begin
  Result := (IndexOf(ACont) >= 0);
end;

function TIntegerList.GetAsString(stDelimiter: String = ','): String;
var
  I: Integer;
begin
  for I:=0 to Count-1 do
  begin
    if I > 0 then
      Result := Result + stDelimiter;
    Result := Result + IntToStr(Items[I]);
  end;
end;

function TIntegerList.GetAsVarArray: Variant;
var
  I: Integer;
begin
  Result := VarArrayCreate([0, Count-1], varInteger);
  for I:=0 to Count-1 do
    Result[I] := Items[I];
end;

procedure TIntegerList.SetAcceptDuplicates(const Value: Boolean);
begin
  FAcceptDuplicates := Value;
  if not Value then
    RemoveDuplicates;
end;

procedure TIntegerList.SetAsString(stValues: String; stDelimiter: String = ',');
var
  I: Integer;
begin
  with TStringList.Create do
  try
    Duplicates := dupIgnore;
    Text := AnsiReplaceText(stValues, stDelimiter, #13#10);
    for I:=0 to Count-1 do
      Self.Add(StrToInt(Strings[I]));
  finally
    Free;
  end;
end;

procedure TIntegerList.RemoveDuplicates;
var
  i, j, Val: Integer;
begin
  i:=0;
  while i < Count do
  begin
    j := i + 1;
    Val := Items[i];
    while j < Count do
      if Items[j] = Val then
        Delete(j)
      else
        inc(j);
    inc(i);
  end;
end;

end.
