unit ListItemTilePainter;

interface

uses
  Windows,
  Graphics,
  ComCtrls,
  ImgList;

type
  TCustomPainter = class
  protected
    FCanvas: TCanvas;
    FItem: TListItem;
    FRect: TRect;
    FTextRect: TRect;
    FTextHeight: Integer;
    procedure InitTextRect; virtual; abstract;
    function TruncateLine(Text: string; Rect: TRect): string;
    function ExcludeBadSymbols(Text: string): string;
    function GetDarkGrayColor: Cardinal;
    procedure DrawSelectionRect;
  public
    constructor Create(AItem: TListItem);
    procedure DrawIcon(ImageList: TCustomImageList; ImageIndex: Integer); overload;
    procedure DrawIcon; overload;
  end;

  //    TListItem   Tile ()
  TListItemTilePainter = class(TCustomPainter)
  private
    procedure InitTextRect; override;
    procedure DrawLine(Line: string; Index: Integer);
    function ExtractOneLine(var Text: string; Width: Integer; Delims: String): string;
  public
    procedure DrawText(Caption, Info: string; Grayed: Boolean);
  end;

  //    TListItem   Report ()
  TListItemReportPainter = class(TCustomPainter)
  private
    procedure InitTextRect; override;
    procedure DrawLine(const ALine: string);
  public
    procedure DrawText(const ACaption: string; AGrayed: Boolean);
  end;

implementation

uses
  SysUtils,
  StrUtils,
  Math;

procedure SplitColor(Color: TColor; var R, G, B: Integer);
begin
  R := Color and $FF;
  G := (Color and $FF00) shr 8;
  B := (Color and $FF0000) shr 16;
end;

function SumColor(R, G, B: Integer): TColor;
begin
  Result := R or (G shl 8) or (B shl 16);
end;

//   
function GetMiddleColor(Col1, Col2: TColor; InPos: Real = 0.5): TColor;
var
  R1, G1, B1,
  R2, G2, B2,
  R, G, B: Integer;

  function GetMid(V1, V2: Integer; InPos: Real): Integer;
  begin
    if V1 < V2 then
      Result := V1 + Round((V2-V1)*InPos)
    else
      if V1 > V2 then
        Result := GetMid(V2, V1, 1-InPos)
      else
        Result := V1;
  end;
  
begin
  if InPos = 0 then
    Result := Col1
  else
  if InPos = 1 then
    Result := Col2
  else
  begin
    SplitColor(ColorToRgb(Col1), R1, G1, B1);
    SplitColor(ColorToRgb(Col2), R2, G2, B2);
    R := GetMid(R1, R2, InPos);
    G := GetMid(G1, G2, InPos);
    B := GetMid(B1, B2, InPos);
    Result := SumColor(R, G, B);
  end;
end;


{ TCustomPainter }

constructor TCustomPainter.Create(AItem: TListItem);
begin
  FItem := AItem;
  FRect := FItem.DisplayRect(drLabel);
  FCanvas := AItem.Owner.Owner.Canvas;
  FTextHeight := FCanvas.TextHeight('|q');
  InitTextRect;
end;

function TCustomPainter.TruncateLine(Text: string; Rect: TRect): string;
//  ,  "..."
var
  chars: array [0..1024] of Char;
begin
  StrPLCopy(@chars, Text, 1024);
  Windows.DrawText(
    FCanvas.Handle,
    @chars,
    Length(Text),
    Rect,
    DT_WORD_ELLIPSIS or DT_MODIFYSTRING or DT_CALCRECT
  );
  Result := PChar(@chars);
end;

procedure TCustomPainter.DrawIcon(ImageList: TCustomImageList;
  ImageIndex: Integer);
var
  R: TRect;
begin
  if not Assigned(ImageList) then
    Exit;
  R := FItem.DisplayRect(drIcon);
  //      ?  ..  
  //  ,    
  //FCanvas.FillRect(R);
  R.Left := (R.Left + R.Right - ImageList.Width) div 2;
  R.Top := (R.Top + R.Bottom - ImageList.Height) div 2;
  //DrawImageListItem(ImageList, ImageIndex, R.Left, R.Top, FCanvas, False);
  ImageList.Draw(FCanvas, R.Left, R.Top, ImageIndex);
end;

procedure TCustomPainter.DrawIcon;
begin
  DrawIcon(
    TListView(FItem.Owner.Owner).LargeImages,
    FItem.ImageIndex
  );
end;

function TCustomPainter.ExcludeBadSymbols(Text: string): string;
begin
  Result := AnsiReplaceStr(Text, #9, ' ');
  while Pos('  ', Result) > 0 do
    Result := AnsiReplaceStr(Result, '  ', ' ');
end;

function TCustomPainter.GetDarkGrayColor: Cardinal;
//  - 
begin
  Result := GetMiddleColor(clGrayText, clWindowText, 0.4);
end;

procedure TCustomPainter.DrawSelectionRect;
//  
var
  ItemRect: TRect;
begin
  ItemRect := FTextRect;
  Dec(ItemRect.Top, 1);
  Dec(ItemRect.Left, 2);
  Inc(ItemRect.Right, 1);
  FCanvas.Brush.Color := ColorToRGB(clHighlight);
  FCanvas.FillRect(ItemRect);
  if FItem.Focused then
    FCanvas.DrawFocusRect(ItemRect);
end;


{ TListItemTilePainter }

function TListItemTilePainter.ExtractOneLine(var Text: string; Width: Integer;
  Delims: String): string;
//     
var
  DelimiterPosition: Integer;
  I: Integer;
  LineWidth: Integer;
begin
  if Width >= FCanvas.TextWidth(Text) then
  begin
    Result := Text;
    Text := '';
    Exit;
  end;
  LineWidth := 0;
  DelimiterPosition := 1;
  I := 1;
  while LineWidth <= Width do
  begin
    if Pos(Text[I], Delims) > 0 then
      DelimiterPosition := I;
    Inc(LineWidth, FCanvas.TextWidth(Text[I]));
    Inc(I);
  end;
  if DelimiterPosition = 1 then
  begin
    Result := Copy(Text, 1, I - 1);
    Delete(Text, 1, I - 1);
  end
  else
  begin
    Result := Copy(Text, 1, DelimiterPosition - 1);
    Delete(Text, 1, DelimiterPosition);
  end;
end;

procedure TListItemTilePainter.InitTextRect;
//     
var
  ItemHeight: Integer;
begin
  FTextRect := FRect;
  ItemHeight := FTextHeight * 2 + 1;
  FTextRect.Top := Max(
    FRect.Top + (FRect.Bottom - FRect.Top - ItemHeight) div 2,
    FRect.Top + 1
  );
  FTextRect.Bottom := Min(FTextRect.Top + ItemHeight, FRect.Bottom);
  FTextRect.Left := FRect.Left + 2;
  FTextRect.Right := FRect.Right - 1;
end;

procedure TListItemTilePainter.DrawLine(Line: string; Index: Integer);
//   
var
  R: TRect;
begin
  R := FTextRect;
  R.Top := R.Top + FTextHeight * Index;
  Windows.DrawText(
    FCanvas.Handle,
    PChar(Line),
    Length(Line),
    R,
    0
  );
end;

procedure TListItemTilePainter.DrawText(Caption, Info: string; Grayed: Boolean);
var
  Line1: string;
  Line2: string;
  bDrawInfo: Boolean;
begin
  Caption := ExcludeBadSymbols(Caption);
  Info := ExcludeBadSymbols(Info);

  //   
  FCanvas.FillRect(FRect);

  //   
  Line1 := ExtractOneLine(Caption, FTextRect.Right - FTextRect.Left, ' '#9);
  bDrawInfo := Caption = '';
  if bDrawInfo then
    Line2 := TruncateLine(Info, FTextRect)
  else
    Line2 := TruncateLine(Caption, FTextRect);

  //  
  FTextRect.Right :=
    FTextRect.Left + Max(FCanvas.TextWidth(Line1), FCanvas.TextWidth(Line2));

  if FItem.Selected then
  begin
    DrawSelectionRect;
    SetBkColor(FCanvas.Handle, ColorToRGB(clHighlight));
    SetTextColor(FCanvas.Handle, ColorToRGB(clHighlightText));
    DrawLine(Line1, 0);
    DrawLine(Line2, 1);
  end
  else
  begin
    if Grayed then
      SetTextColor(FCanvas.Handle, GetDarkGrayColor);
    DrawLine(Line1, 0);
    if bDrawInfo then
      SetTextColor(FCanvas.Handle, ColorToRGB(clGrayText));
    DrawLine(Line2, 1);
  end;
end;


{ TListItemReportPainter }

procedure TListItemReportPainter.InitTextRect;
//     
begin
  FTextRect := FRect;
  FTextRect.Top := FRect.Top + 2;
  FTextRect.Left := FRect.Left + 2;
  FTextRect.Right := FRect.Right - 1;
end;

procedure TListItemReportPainter.DrawLine(const ALine: string);
//   
var
  R: TRect;
begin
  R := FTextRect;
  //R.Top := R.Top + FTextHeight * Index;
  Windows.DrawText(
    FCanvas.Handle,
    PChar(ALine),
    Length(ALine),
    R,
    0
  );
end;

procedure TListItemReportPainter.DrawText(const ACaption: string; AGrayed: Boolean);
var
  Line1: string;
begin
  Line1 := ExcludeBadSymbols(ACaption);
  Line1 := TruncateLine(Line1, FTextRect);

  //   
  FCanvas.FillRect(FRect);

  //  
  FTextRect.Right :=
    FTextRect.Left + FCanvas.TextWidth(Line1);

  if FItem.Selected then
  begin
    DrawSelectionRect;
    SetBkColor(FCanvas.Handle, ColorToRGB(clHighlight));
    SetTextColor(FCanvas.Handle, ColorToRGB(clHighlightText));
    DrawLine(Line1);
  end
  else
  begin
    if AGrayed then
      SetTextColor(FCanvas.Handle, GetDarkGrayColor);
    DrawLine(Line1);
  end;
end;


end.

