unit MiscGUI;

interface

uses
  Controls,
  Graphics,
  Forms,
  ImgList;


procedure PlaceFrameToControl(AFrame: TForm; AControl: TWinControl); overload;
procedure PlaceFrameToControl(AFrame: TFrame; AControl: TWinControl); overload;
procedure StretchBitMap(ABitmap: TBitmap; ANewWidth, ANewHeight: Integer);
procedure BitmapLoadFromResource(AInstance: THandle; const AResName: string;
  AResType: PChar; out ABitmap: TBitmap);
procedure ConvertToHighColor(ImageList: TCustomImageList);
function FilteredGUID(const s: string): string;
function GetAgreeProgressIconIndex(const AProgressValue: string): Integer;
function AddStringWithSeparator(const s1, s2: string; Separator: string): string;
function ConvertToUserTitle(const User, FullName: string; ShowUser: Boolean = True): string;
function DateToStrEx(const DateTime: TDateTime): string;
function IsNull(Date: TDateTime): Boolean; overload;
function IsNull(const Str: String): Boolean; overload;
function SameTextIgnoreCase(const s1, s2: string): Boolean;
function IsPGS(const AppTitle: string): Boolean;
function IsORD(const AppTitle: string): Boolean;


implementation

uses
  Classes,
  SysUtils,
  CommCtrl,
  StrUtils;


procedure PlaceFrameToControl(AFrame: TForm; AControl: TWinControl);
//     
begin
  AFrame.BorderStyle := bsNone;
  AFrame.Align := alClient;
  AFrame.Width := AControl.ClientWidth;
  AFrame.Height := AControl.ClientHeight;
  AFrame.Parent := AControl;
  AFrame.Visible := true;
end;

procedure PlaceFrameToControl(AFrame: TFrame; AControl: TWinControl);
//     
begin
  AFrame.Align := alClient;
  AFrame.Parent := AControl;
  AFrame.Visible := true;
end;

procedure StretchBitMap(ABitmap: TBitmap; ANewWidth, ANewHeight: Integer);
var
  BM_Stretch: TBitmap;
begin
  //  -  
  if (ABitmap.Width = ANewWidth) and (ABitmap.Height = ANewHeight) then
    Exit;
  BM_Stretch := TBitmap.Create;
  try
    BM_Stretch.Width := ANewWidth;
    BM_Stretch.Height := ANewHeight;
    BM_Stretch.Canvas.StretchDraw(BM_Stretch.Canvas.Cliprect, ABitmap);
    ABitmap.Assign(BM_Stretch);
  finally
    FreeAndNil(BM_Stretch);
  end;
end;

procedure BitmapLoadFromResource(AInstance: THandle; const AResName: string;
  AResType: PChar; out ABitmap: TBitmap);
var
  AStream: TStream;
begin
  AStream := TResourceStream.Create(AInstance, AResName, AResType);
  try
    ABitmap.LoadFromStream(AStream);
  finally
    AStream.Free;
  end;
end;

procedure ConvertToHighColor(ImageList: TCustomImageList);
// To show smooth images we have to convert the image list from 16 colors
// to high color.
var
  IL: TCustomImageList;
begin
  // Have to create a temporary copy of the given list, because the list is cleared on handle creation.
  IL := TCustomImageList.Create(nil);
  IL.Assign(ImageList);
  with ImageList do
    Handle := ImageList_Create(Width, Height, ILC_COLOR32 or ILC_MASK, Count, AllocBy);
  ImageList.Assign(IL);
  IL.Free;
end;

function RPos(const Find, Search: string): Integer;
//     
//  FindS   SrcS, ..   .
//   ,    .

  function InvertS(const S: string): string;
  //   S
  var
    i, Len: Integer;
  begin
    Len := Length(S);
    SetLength(Result, Len);
    for i := 1 to Len do
      Result[i] := S[Len - i + 1];
  end;

var
  ps: Integer;
begin
  // :    
  //  ''   '  '.
  //     
  //  ''  '  ',
  //        
  //  Pos(Substr, S: string): string;
  //   Substr    S, 
  //      ,
  //    .
  ps := Pos(InvertS(Find), InvertS(Search));
  //       
  //  ,   
  if ps <> 0 then
    Result := Length(Search) - Length(Find) - ps + 2
  else
    Result := 0;
end;

function Before(const Search, Find: string): string;
// before()   ,  
//    Find   Search. 
// Find  ,   Search.
var
  index: byte;
begin
  index := Pos(Find, Search);
  if index = 0 then
    Result := Search
  else
    Result := Copy(Search, 1, index - 1);
end;

function Inside(const Search, Front, Back: string): string;
// inside()  ,     Front ... Back
var
  Index, Len: Integer;
begin
  Index := RPos(Front, Before(Search, Back));
  Len := Pos(Back, Search);
  if (Index > 0) and (Len > 0) then
    Result := Copy(Search, Index + 1, Len - (Index + 1))
  else
    Result := '';
end;

function FilteredGUID(const s: string): string;
//  GUID  
var
  stGUID: string;
begin
  stGUID := Inside(s, '{', '}');
  if stGUID = '' then
    Result := s
  else
    Result := ReplaceStr(s, '{' + stGUID + '}', '');
end;

function GetAgreeProgressIconIndex(const AProgressValue: string): Integer;
begin
  if AProgressValue = '' then
  begin
    Result := -1;
    Exit;
  end;

  Result := StrToIntDef(AProgressValue, -1);
end;

function AddStringWithSeparator(const s1, s2: string;
  Separator: string): string;
begin
  if s1 <> '' then
  begin
    if s2 <> '' then
      Result := s1 + Separator + s2
    else
      Result := s1;
  end
  else
    Result := s2;
end;

function ConvertToUserTitle(const User, FullName: string; ShowUser: Boolean = True): string;
//   User/FullName    ".. (DOMAIN\Kotov)".
//  ShowUser = False    
begin
  Result := '';

  if FullName <> '' then
  begin
    if ShowUser then
      Result := Format('%s (%s)', [FullName, User])
    else
      Result := FullName;
  end
  else
  if User <> '' then
    Result := User;
end;

function DateToStrEx(const DateTime: TDateTime): string;
begin
  if IsNull(DateTime) then
    Result := ''
  else
    Result := DateToStr(DateTime);
end;

function IsNull(Date: TDateTime): Boolean; overload;
begin
  Result := Trunc(Date) = 0;
end;

function IsNull(const Str: string): Boolean; overload;
begin
  Result := Length(Trim(Str)) = 0;
end;

function SameTextIgnoreCase(const s1, s2: string): Boolean;
//    ,     " " 
begin
  Result := SysUtils.AnsiCompareText(s1, s2) = 0
end;

function IsPGS(const AppTitle: string): Boolean;
begin
 Result := SameTextIgnoreCase(AppTitle, ':');
end;

function IsORD(const AppTitle: string): Boolean;
begin
 Result := SameTextIgnoreCase(AppTitle, ':');
end;

end.
