unit Plugin;

interface

uses
  Windows,
  SysUtils,

  PluginIntf,
  PluginIntfWrapper;


type
  TPlugin = class(TInterfacedObject, IPlugin, IPluginCoolCommands, IPluginEvents)
  private
    FLastError: WideString;
    FAppProvider: TAppProvider;
    procedure ShowException(const AMessage: string);
    procedure DlgTaskCreateShow;
    procedure DlgTaskEditShow;
    procedure TaskDelete;
  private
    // IPlugin
    function GetVersion(): Integer; safecall;
    function GetLastError(): WideString; safecall;
    procedure Init(AppProvider: IPluginAppProvider); safecall;
    // IPluginCoolCommands
    procedure InitCoolCommands(ACCPanel: IPluginCoolCommandsPanel); safecall;
    procedure CallCoolCommand(AIndex: Integer); safecall;
    // IPluginEvents
    function AfterCreateDocument(ADocument: IPluginPDMVersion; AFile: WideString): TCallResult; safecall;
    function AfterEditObject(AParent, AObject: IPluginPDMVersion; AIdLink: Integer;
      ALinkType, ANewProduct, ANewState: WideString): TCallResult; safecall;
    function CheckAttachment(Version: IPluginPDMVersion; out RejectReason: WideString): TCallResult; safecall;
    function BeforeUnloadingPlugin: TCallResult; safecall;
    function BeforCreateTaskInPF(AVersions: IPluginPDMVersions; out PluginTask: WordBool): TCallResult; safecall;
  public
    constructor Create;
    destructor Destroy; override;
  end;


function LoadPlugin(): IPlugin; safecall;


implementation

uses
  Dialogs,
  Forms,

  DlgTaskCreate,
  DlgTaskEdit;


function LoadPlugin(): IPlugin;
begin
  Result := TPlugin.Create;
end;


{ TPlugin }

constructor TPlugin.Create;
begin
end;

destructor TPlugin.Destroy;
begin
  //   0  Application.Handle
  Application.Handle := 0;
  if Assigned(FAppProvider) then
    FreeAndNil(FAppProvider);
  inherited;
end;

procedure TPlugin.ShowException(const AMessage: string);
begin
  Application.MessageBox(
    PChar(AMessage),
    PChar(' '),
    MB_ICONERROR or MB_OK or MB_TOPMOST);
end;

function TPlugin.GetLastError: WideString;
begin
  Result := FLastError;
end;

function TPlugin.GetVersion: Integer;
begin
  Result := PLUGIN_VERSION;
end;

procedure TPlugin.Init(AppProvider: IPluginAppProvider);
begin
  try
    FAppProvider := TAppProvider.Create(AppProvider);
  except
    on E: Exception do
    begin
      ShowException(E.Message);
    end;
  end;
end;

procedure TPlugin.InitCoolCommands(ACCPanel: IPluginCoolCommandsPanel);

  function I_am_(const UserName: string): Boolean;
  begin
    Result := SameText(FAppProvider.AppCache.GetUserName, UserName);
  end;

  function CanEdit: Boolean;
  var
    Tasks: IPluginTasks;
    Task: IPluginTask;
  begin
    Result := False;

    Tasks := FAppProvider.GetSelectedTasks;
    if Tasks.Count = 1 then
    begin
      Task := Tasks.Task(0);
      Result := I_am_(Task.Author) and
                (Task.State in [tsPlanned, tsSend, tsInWork]) and
                (Task.ProcessKind = bpkTask);
    end;
  end;

begin
  try
    //    ,    
    ACCPanel.AddCommand2(cSpaceTasks, cGroupTask, ' ', 'IMAGE_NEWTASK', 1);
    if CanEdit then
    begin
      ACCPanel.AddCommand2(cSpaceTasks, cGroupTask, ' ', 'IMAGE_EDIT', 2);
      ACCPanel.AddCommand2(cSpaceTasks, cGroupTask, ' ', '', 3);
    end;
  except
    on E: Exception do
    begin
      ShowException(E.Message);
    end;
  end;
end;

function TPlugin.AfterCreateDocument(ADocument: IPluginPDMVersion;
  AFile: WideString): TCallResult;
begin
  Result := crOk;
end;

function TPlugin.AfterEditObject(AParent, AObject: IPluginPDMVersion;
  AIdLink: Integer; ALinkType, ANewProduct, ANewState: WideString): TCallResult;
begin
  Result := crOk;
end;

function TPlugin.BeforCreateTaskInPF(AVersions: IPluginPDMVersions;
  out PluginTask: WordBool): TCallResult;
begin
  PluginTask := False;
  Result := crOk;
end;

function TPlugin.BeforeUnloadingPlugin: TCallResult;
begin
  Result := crOk;
end;

procedure TPlugin.CallCoolCommand(AIndex: Integer);
begin
  try
    //  AppHandle      .
    // !     0  Application.Handle
    Application.Handle := FAppProvider.GetAppHandle;

    case AIndex of
      1: DlgTaskCreateShow;
      2: DlgTaskEditShow;
      3: TaskDelete;
    end;
  except
    on E: Exception do
    begin
      ShowException(E.Message);
    end;
  end;
end;

procedure TPlugin.DlgTaskCreateShow;
var
  DlgTaskCreate: TfrmDlgTaskCreate;
begin
  DlgTaskCreate := TfrmDlgTaskCreate.Create(nil, FAppProvider);
  try
    DlgTaskCreate.ShowModal;
  finally
    FreeAndNil(DlgTaskCreate);
  end;
end;

procedure TPlugin.DlgTaskEditShow;
var
  DlgTaskEdit: TfrmDlgTaskEdit;
begin
  DlgTaskEdit := TfrmDlgTaskEdit.Create(nil, FAppProvider);
  try
    DlgTaskEdit.ShowModal;
  finally
    FreeAndNil(DlgTaskEdit);
  end;
end;

procedure TPlugin.TaskDelete;
var
  Task: IPluginTask;
begin
  Task := FAppProvider.GetSelectedTasks.Task(0);
  FAppProvider.AppWorkFlow.DeleteProcess(Task.RouteId);
  FAppProvider.RefreshTasks(Task.RouteId);
end;

function TPlugin.CheckAttachment(Version: IPluginPDMVersion; out RejectReason: WideString): TCallResult;
begin
  //         
  if not FAppProvider.AppCache.IsDocument(Version.TypeObj) then
  begin
    RejectReason := '    ( DemoCreateTask)';
    Exit;
  end;

  if Version.AccessLevel <> alAdministration then
  begin
    RejectReason := Format('     "%s" ( DemoCreateTask)', [
                          Version.Title
                          ]);
    Exit;
  end;
end;

end.
